#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug  1 14:37:04 2019

@author: jtu
"""
import math
import numpy as np
from scipy import interpolate
import matplotlib.pyplot as plt

import Cs_Rind_Lt
import Cs_Rind

#-------------------------------------------------------------------------------------
#------------ Calculate Q-factor -----------------------------------------------------
#-------------------------------------------------------------------------------------
def calc_Q(Va,freq):
    #performing interpolate to increase frequency resolution so as to increase accuracy
    #of the Q calculation
    maxfq=max(freq)
    fqin=np.linspace(min(freq), maxfq, len(freq)*100)
    fq=np.flipud(fqin)

    Va_interp=interpolate.interp1d(freq, Va, kind='linear')
    Va1=Va_interp(fq)

    ipeak=np.argmax(Va1)
    V70p=np.max(Va1)/np.sqrt(2)

    iL1=-1
    iL=-1
    iL2=-1
    #original order is from high to low frequency - needed to be reversed
    for i in range(ipeak, len(fq)):
        if Va1[i] > V70p:                       #  +            ipeak
            iL2=i                              #   +        
        elif Va1[i] == V70p:                    #     +         iL2
            iL=i                               #       +    ------- V70p (iL)
            break                              #         +     iL1
        else:                                  #          +
            iL1=i                              #            +  len(freq)
            break

    for i in range(ipeak, iL1):
        if Va1[i+1] >= Va1[i]:
            return -2.0                       #Va non monotonous dcrease

    if iL == -1:
        if iL2 >= ipeak and iL1 > iL2:
            fhL= (fq[iL1]*(Va1[iL2]-V70p)+fq[iL2]*(V70p-Va1[iL1])) \
                /(Va1[iL2]-Va1[iL1])
        else:
            return -1.0
    else:
        fhL=fq[iL]

    iH1=-1
    iH=-1
    iH2=-1
    for i in range(0, ipeak+1):
        if Va1[i] < V70p:                       #           +  ipeak
            iH1=i                              #          +
        elif Va1[i] == V70p:                    #        +     iH2
            iH=i                               #       +   --------V70p (iH)
            break                              #      +       iH1
        else:                                  #    +
            iH2=i                              #   +          0
            break

    for i in range(iH1,ipeak):
        if Va1[i+1] <= Va1[i]:
            return -2.0                       #Va non monotonous increase

    if iH == -1:
        if iH1 >= 0 and iH2 > iH1:
            fhH= (fq[iH1]*(Va1[iH2]-V70p)+fq[iH2]*(V70p-Va1[iH1])) \
                /(Va1[iH2]-Va1[iH1])
        else:
            return -1.0
    else:
        fhH=fq[iH]

    Q=freq[np.argmax(Va)]/(fhH-fhL)

    return Q

#############################################################################
#---------- Compute antenna Ra and Ca for both branches ------------------
#############################################################################
def Ca_Ra(entv, Q, Va1, Va2, freq):
    pi=math.pi
    pi2=2.0*pi
    #rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    Cs1p, RLp, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Cs1m, RLm, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])
    Cip=entv[2]*1.0e-12+Cs1p
    Cim=entv[5]*1.0e-12+Cs1m

    Omegar=pi2*freq[nfsp]
    XL=Omegar*Ltp
    XC=1.0/(Omegar*Cip)

    Rtotp=XL/Q[0]
    Itotp=entv[0]/Rtotp
    Rp=Rtotp-RLp

    temp=(XC-XL)**2+Rp*Rp
    Rap=Rp*XC*XC/temp

    Cap=Cip*temp/((XC-XL)*XC-temp)

    Iap=Itotp*np.sqrt((Rp)/Rap) # Va1[nfsp]/np.sqrt(Rap*Rap+1.0/(Omegar*Cap)**2)

    Omegar=pi2*freq[nfsm]
    XL=Omegar*Ltm
    XC=1.0/(Omegar*Cim)

    Rtotm=XL/Q[1]
    Itotm=entv[3]/Rtotm
    Rm=Rtotm-RLm

    temp=(XC-XL)**2+Rm*Rm
    Ram=Rm*XC*XC/temp

    Cam=Cim*temp/((XC-XL)*XC-temp)
 
    Iam=Itotm*np.sqrt(Rm/Ram) #Va2[nfsm]/np.sqrt(Ram*Ram+1.0/(Omegar*Cam)**2)
 
    return Rap, Ram, Cap*1.0e12, Cam*1.0e12, Iap, Iam

"""""
def Ca_Ra_symmetric(entv, Q, Va1, phi1, Va2, phi2, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfsp=np.argmax(Va1)

    phip=phi1[nfsp]*rad
    phim=phi2[nfsp]*rad

    #see note Resonance_solutionOf-Ra&Ca.docx
    #for entire dipole
    Cs, RL=Cs_Rind.Cs_RL(entv[1], freq[nfsp])

    #two C1 (including Cstray) capacitance connected in series
    Ci=0.5*(entv[2]*1.0e-12+Cs)

    Omegar=pi2*freq[nfsp]
    XL=2.0*Omegar*entv[1]      #sum of two TATUs inductance
    XC=1.0/(Omegar*Ci)

    R1=XL/Q[0]-2.0*RL     #effect of internal resistances from both TATUs

    temp=(XC-XL)**2+R1*R1
    Rap=R1*XC*XC/temp
    Cap=Ci*temp/((XC-XL)*XL-R1*R1)

    Va2= (Va1[nfsp]**2+Va2[nfsp]**2 - 2.0*Va1[nfsp]*Va2[nfsp]*math.cos(phip-phim))

    Iap=np.sqrt(Va2/(Rap*Rap+1.0/(Omegar*Cap)**2))

    Ram=Rap
    Cam=Cap
    Iam=Iap

    #current divided by a factor of sqrt(2), then close to measured current
    return Rap, Ram, Cap*1.0e12, Cam*1.0e12, Iap, Iam
    """

#############################################################################
#---------- Compute antenna Ra and Ca for one branche on ------------------
#############################################################################
def Ca_Ra_asymmetr(entv, Q, Va1, Va2, freq):
    pi=math.pi
    pi2=2.0*pi

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    #see note Resonance_solutionOf-Ra&Ca.docx & and equivalent circuit for assymetric case
    #TATU minus has only a Cstray
    Csp, RLp, _ =Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Csm=125.2697113202511e-12

    Cip=entv[2]*1.0e-12+Csp

    Omegar=pi2*freq[nfsp]
    XL=Omegar*entv[1]
    XC=1.0/(Omegar*Cip)
    Rd=XL/Q[0]-RLp

    temp=(XC-XL)**2+Rd*Rd
    Rap=XC*XC*Rd/temp

    # calculated Ca by Ia = Va1/|Ra-i/(Omegar*Ca)-i/(Omegar*Csm)| = Va2/|1/(Omegar*Csm)|
    Xcb=np.sqrt((Va1[nfsp]/(Va2[nfsm]*Omegar*Csm))**2-Rap*Rap)
    Cap=1.0/(Xcb*Omegar-1.0/Csm)

    Iap=Va2[nfsm]*Omegar*Csm

    # The resistance and capacitance of TATU minus is halfed and doubled
    Ram=Rap/2.0
    Cam=Cap*2.0

    Iam=Iap

    return Rap, Ram, Cap*1.0e12, Cam*1.0e12, Iap, Iam

def Ca_Ra_comb(entv, entv3, Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfq=len(freq)
    Vat=np.zeros(nfq)

    for i in range(nfq):
        phip=phi1[i]*rad
        phim=phi2[i]*rad

        cosp=np.cos(phip)
        sinp=np.sin(phip)
        cosm=np.cos(phim)
        sinm=np.sin(phim)

        Vat[i]=np.sqrt((Va1[i]*cosp-Va2[i]*cosm)**2+(Va1[i]*sinp-Va2[i]*sinm)**2)
        
    Q=calc_Q(Vat,freq)
    if Q <= 0.0:
        return -1, -1, Vat, -1, -1

    nfs=np.argmax(Vat)

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfs])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfs])
    C1p=entv[2]+Cs1p*1.0e12
    C1m=entv[5]+Cs1m*1.0e12

    Lt=Ltp+Ltm
    Cint=C1p*C1m/(C1p+C1m)
    Rint=R1p+R1m

    Omegar=pi2*freq[nfs]
    XL=Omegar*Lt

    Ca=1.0e12/(Omegar**2*Lt)-Cint

    Rp=XL/Q-Rint
    Ra=Rp*((Cint+Ca)/Ca)**2

    Ia=Vat[nfs]/np.abs(complex(Ra,-1.0e12/(Omegar*Ca)))

    return Ra, Ca, Vat, Ia, Q

def Ca_Ra1(entv, entv3, Q, Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])
    C1p=entv[2]*1.0e-12+Cs1p
    C1m=entv[5]*1.0e-12+Cs1m

    Omega=pi2*freq[nfsp]

    Lt=Ltp+Ltm
    Ct=1.0/(Omega**2*Lt)
    Ca=Ct-C1p*C1m/(C1p+C1m)

    Ya=1.0/(Omega*Ca)
    Rt=Omega*Lt/Q[0]

    Ra=(Rt-(R1p+R1m))*(Ct/Ca)**2

    phip=phi1[nfsp]*rad
    phim=phi2[nfsm]*rad
    Vapr=Va1[nfsp]*np.cos(phip)
    Vapi=Va1[nfsp]*np.sin(phip)
    Vamr=Va2[nfsm]*np.cos(phim)
    Vami=Va2[nfsm]*np.sin(phim)

    Ia=complex(Vapr-Vamr,Vapi-Vami)/complex(Ra,-Ya)

    return Ra, Ca*1.0e12, Ia
