#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Aug  1 14:37:04 2019

@author: jtu
"""
import math
import numpy as np
import Cs_Rind_Lt

def calc_Q(Va,freq):
    ipeak=np.argmax(Va)
    V70p=0.7071*np.max(Va)

    iL1=-1
    iL=-1
    iL2=-1
    #original order is from high to low frequency - needed to be reversed
    for i in range(ipeak, len(freq)):
        if Va[i] > V70p:                       #  +            ipeak
            iL2=i                              #   +        
        elif Va[i] == V70p:                    #     +         iL2
            iL=i                               #       +    ------- V70p (iL)
            break                              #         +     iL1
        else:                                  #          +
            iL1=i                              #            +  len(freq)
            break

    for i in range(ipeak, iL1):
        if Va[i+1] >= Va[i]:
            return -2.0                       #Va non monotonous dcrease

    if iL == -1:
        if iL2 >= ipeak and iL1 > iL2:
            fhL= (freq[iL1]*(Va[iL2]-V70p)+freq[iL2]*(V70p-Va[iL1])) \
                /(Va[iL2]-Va[iL1])
        else:
            return -1.0
    else:
        fhL=freq[iL]

    iH1=-1
    iH=-1
    iH2=-1
    for i in range(0, ipeak+1):
        if Va[i] < V70p:                       #           +  ipeak
            iH1=i                              #          +
        elif Va[i] == V70p:                    #        +     iH2
            iH=i                               #       +   --------V70p (iH)
            break                              #      +       iH1
        else:                                  #    +
            iH2=i                              #   +          0
            break

    for i in range(iH1,ipeak):
        if Va[i+1] <= Va[i]:
            return -2.0                       #Va non monotonous increase

    if iH == -1:
        if iH1 >= 0 and iH2 > iH1:
            fhH= (freq[iH1]*(Va[iH2]-V70p)+freq[iH2]*(V70p-Va[iH1])) \
                /(Va[iH2]-Va[iH1])
        else:
            return -1.0
    else:
        fhH=freq[iH]

    #fhL=np.interp(V70p, Va[0:ipeak], freq[0:ipeak])
    #Vrev=np.zeros(imax-ipeak+1)
    #frev=np.zeros(imax-ipeak+1)
    #for i in range(0, imax-ipeak+1):
    #    Vrev[i]=Va[imax-i]
    #    frev[i]=freq[imax-i]
    #fhH=np.interp(V70p, Vrev, frev)

    Q=freq[ipeak]/(fhH-fhL)
    
    #if k==2000:
    #    print(Q, fhH, fhL, 'old')

    return Q

def Ca_Ra(entv, entv3, Q, Va1, phi1, Va2, phi2, freq):
    pi=math.pi
    pi2=2.0*pi
    #rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    #phip=phi1[nfsp]*rad
    #phim=phi2[nfsm]*rad

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])
    Cip=entv[2]*1.0e-12+Cs1p
    Cim=entv[5]*1.0e-12+Cs1m

    Omegar=pi2*freq[nfsp]
    XL=Omegar*Ltp
    XC=1.0/(Omegar*Cip)

    Rp=XL/Q[0]-R1p

    temp=(XC-XL)**2+Rp*Rp
    Rap=Rp*XC*XC/temp

    Cap=Cip*temp/((XC-XL)*XC-temp)
    #Cap=Cs1p*Cas/(Cs1p-Cas)

    Iap=Va1[nfsp]/np.sqrt(Rap*Rap+1.0/(Omegar*Cap)**2)
    #Iap=Va1[nfsp]*complex(np.cos(phip),np.sin(phip))/complex(Rap,-1.0/(Omegar*Cap))
 
    Omegar=pi2*freq[nfsm]
    XL=Omegar*Ltm
    XC=1.0/(Omegar*Cim)

    Rp=XL/Q[1]-R1m

    temp=(XC-XL)**2+Rp*Rp
    Ram=Rp*XC*XC/temp

    Cam=Cim*temp/((XC-XL)*XC-temp)
    #Cam=Cs1m*Cas/(Cs1m-Cas)

    Iam=Va2[nfsm]/np.sqrt(Ram*Ram+1.0/(Omegar*Cam)**2)
    #Iam=Va2[nfsp]*complex(np.cos(phim),np.sin(phim))/complex(Ram,-1.0/(Omegar*Cam))

    return Rap, Ram, Cap*1.0e12, Cam*1.0e12, Iap, Iam

def Ca_Ra_comb(entv, entv3, Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfq=len(freq)
    Vat=np.zeros(nfq)

    for i in range(nfq):
        phip=phi1[i]*rad
        phim=phi2[i]*rad

        cosp=np.cos(phip)
        sinp=np.sin(phip)
        cosm=np.cos(phim)
        sinm=np.sin(phim)

        Vat[i]=np.sqrt((Va1[i]*cosp-Va2[i]*cosm)**2+(Va1[i]*sinp-Va2[i]*sinm)**2)
        
    Q=calc_Q(Vat,freq)
    if Q <= 0.0:
        return -1, -1, Vat, -1, -1

    nfs=np.argmax(Vat)

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfs])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfs])
    C1p=entv[2]+Cs1p*1.0e12
    C1m=entv[5]+Cs1m*1.0e12

    Lt=Ltp+Ltm
    Cint=C1p*C1m/(C1p+C1m)
    Rint=R1p+R1m

    Omegar=pi2*freq[nfs]
    XL=Omegar*Lt

    Ca=1.0e12/(Omegar**2*Lt)-Cint

    Rp=XL/Q-Rint
    Ra=Rp*((Cint+Ca)/Ca)**2

    Ia=Vat[nfs]/np.abs(complex(Ra,-1.0e12/(Omegar*Ca)))

    return Ra, Ca, Vat, Ia, Q

def Ca_Ra1(entv, entv3, Q, Va1, phi1, Va2, phi2, Va31, phi31, Va32, phi32, freq):
    pi=math.pi
    pi2=2.0*pi
    rad=pi/180.0

    nfsp=np.argmax(Va1)
    nfsm=np.argmax(Va2)

    Cs1p, R1p, Ltp=Cs_Rind_Lt.Cs_RL_Lt(entv[1], freq[nfsp])
    Cs1m, R1m, Ltm=Cs_Rind_Lt.Cs_RL_Lt(entv[4], freq[nfsm])
    C1p=entv[2]*1.0e-12+Cs1p
    C1m=entv[5]*1.0e-12+Cs1m

    Omega=pi2*freq[nfsp]

    Lt=Ltp+Ltm
    Ct=1.0/(Omega**2*Lt)
    Ca=Ct-C1p*C1m/(C1p+C1m)

    Ya=1.0/(Omega*Ca)
    Rt=Omega*Lt/Q[0]

    Ra=(Rt-(R1p+R1m))*(Ct/Ca)**2

    phip=phi1[nfsp]*rad
    phim=phi2[nfsm]*rad
    Vapr=Va1[nfsp]*np.cos(phip)
    Vapi=Va1[nfsp]*np.sin(phip)
    Vamr=Va2[nfsm]*np.cos(phim)
    Vami=Va2[nfsm]*np.sin(phim)

    Ia=complex(Vapr-Vamr,Vapi-Vami)/complex(Ra,-Ya)

    return Ra, Ca*1.0e12, Ia
