import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import ant_impedance
import Cs_Rind_Lt
import days_of_year
import binned_values
import moving_average

#####################################################################################
#####################################################################################
def plot_Ia_freq(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    V0, comb):

    print('Read calculated antenna parameters ...')
    fdir=fpath.strip()
    inpflag=inpflag.strip()
    if fdir[len(fdir)-1] == '/':
        if comb == 'Yes':
            datafile=fdir+'All-Bands_InOut_data_comb.dat'
        else:
            datafile=fdir+'All-Bands_InOut_data.dat'
    else:
        if comb == 'Yes':
            datafile=fdir+'/All-Bands_InOut_data_comb.dat'
        else:
            datafile=fdir+'/All-Bands_InOut_data.dat'

    with open(datafile,'r') as dataf:
        dataf.readline()
        dataf.readline()

        fl=dataf.readlines()

        dataf.close()

    print('Finished reading data')

    nrecd=len(fl)

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,1.95),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=48)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.54]
    yB=[0.63,0.63]
    wh=[0.44,0.44]
    ht=[0.26,0.26]

    if comb == 'Yes':
        ndata=1
        nplt=1
    else:
        ndata=2
        nplt=2

    axes=[]

    for i in range(nplt):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(nplt):
            axes[i].spines[ax].set_linewidth(0.3)

    LL=[0.0375, 0.3015, 2.1515]

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=''
    if V0 == 9696:
        sL1=sL1+' V1 >= 96 V'
    elif V0 == 8080:
        sL1=sL1+' V1 >= 80 V'

    if bname == 'All-Bands':
        if comb == 'Yes':
            minIa=0.1
            maxIa=100
            if V0 == 8080:
                minIa=0.1
                maxIa=300
        else:
            minIa=0.03
            maxIa=60
            if V0 == 8080:
                minIa=0.1
                maxIa=50
    elif bname == 'High-Band':
        minIa=0.2
        maxIa=3.0
    elif bname == 'Mid-Band':
        minIa=0.3
        maxIa=40.0

    date=[]
    fres=[[],[]]
    Ia=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()
        datet=a[0]
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 9696 and float(a[2]) < 96.0:
            print('Driving voltage is < ',96)
            diff=diff+1
            continue
        elif V0 == 8080 and float(a[2]) < 80.0:
            print('Driving voltage is < ',80)
            diff=diff+1
            continue

        if bname == 'High-Band':
            if L1 > 8.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Mid-Band':
            if L1 < 0.02*LL[1] or L1 > 5.0*LL[0]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue
        elif bname == 'Low-Band':
            if L1 < 0.5*LL[2]:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+ \
                    ' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpflag != 'InOut':
            if inpflag == 'In': #skip if not in plasmasphere
                if inpl == 0 or inpl == 2 or inpl == 3:
                    print('Out of the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            else:
                if inpl == 1:  #skip if in the plasmasphere
                    print('In the plasmasphere:','{:2d}'.format(inpl)+ \
                        ' '+datet+' '+a[1])
                    diff=diff+1
                    continue

        if comb == 'Yes':
            fqp=float(a[18])
            Ia1=float(a[22])

            #if (fqp/1.0e3 > 3.8 and fqp/1.0e3 <6.0):
            #    diff=diff+1
            #    continue

            fqm=fqp
            Ia2=Ia1
        else:
            fqp=float(a[5])
            Ia1=float(a[8])                        #Ra in Ohm
            fqm=float(a[16])
            Ia2=float(a[19])

            #if (fqp > 6.0e3 and Ra1 > 2.5e4) or (fqm > 6.0e3 and Ra2 > 2.5e4):
            #    diff=diff+1
            #    continue

        date.append(datet)

        fres[0].append(fqp/1.0e3)          #frequency in kHz
        Ia[0].append(Ia1)                  #current in mA

        fres[1].append(fqm/1.0e3)
        Ia[1].append(Ia2)

    #end 'for loop' for calculating antenna Ra

    if diff > 0:
        print(diff,'data out of',nrecd,'not used in analysis')

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    elif bname == 'High-Band':
        xscale='linear'
        fmin=23
        fmax=32
    else:
        xscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    print('maximum of Ia ', max(Ia[0]))
    for i in range(ndata):
        #--- determine y min & max
        Iamin=min(Ia[i])
        Iamax=max(Ia[i])
        axes[i].scatter(fres[i],Ia[i],color='blue',s=0.1, marker='s')

        #----- set axis properties for Ra ------
        axes[i].set_xlim(fmin,fmax)
        axes[i].set_xscale(xscale)
        axes[i].set_yscale('log')
        #axes[i].set_ylim(minIa,maxIa)
        axes[i].tick_params(axis='both',which='major',length=1.1,width=0.4, \
            pad=1,labelsize=2.5,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[i].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')

        if bname == 'All-Bands':
            axes[i].set_xticks([2.5,5,10,20,30,40])
            axes[i].set_xticklabels(['2.5','5','10','20','30','40'])

        axes[i].set_xlabel('Resonance Frequency (kHz)',fontsize=2.7, \
            labelpad=0.7)

    axes[0].set_ylabel(r'I$_a$ (mA)',fontsize=2.7,labelpad=0.7)

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' Inside Nominal Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' Outside Plasmasphere'
    else:
        txt=sL1+' All Regions'

    if comb == 'No':
        # make x-axis tick lables invisible for upper panel
        axes[1].set_yticklabels([])

        xloc=1.0
        axes[0].text(0,1.04,'Y+ 1st Harmonic',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
        axes[1].text(1,1.04,'Y- 1st Harmonic',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[1].transAxes)
    else:
        xloc=0.5

    axes[0].text(xloc,1.12,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(xloc,1.03,txt,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)

    return canv, fig, toolbar
