import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import ant_impedance
import days_of_year
import binned_values
import moving_average

#----------- plot Ra, Ca, and Prad for Y + In vs Out plasmasphere -----------------
def plot_comp_In_Out(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, \
    fl, V0, comb):

    if len(fl) == 0:
        print('Read calculated antenna parameters ...')
        fdir=fpath.strip()
        inpflag=inpflag.strip()
        if fdir[len(fdir)-1] == '/':
            datafile=fdir+'All-Bands_InOut'+'_data.dat'
        else:
            datafile=fdir+'/All-Bands_InOut'+'_data.dat'

        with open(datafile,'r') as dataf:
            dataf.readline()
            dataf.readline()

            fl=dataf.readlines()

            dataf.close()

        print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,2.0),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.08,0.08,0.08,0.49,0.49,0.49]
    yB=[0.68,0.46,0.24,0.68,0.46,0.24]
    wh=[0.38,0.38,0.38,0.38,0.38,0.38]
    ht=[0.20,0.20,0.20,0.20,0.20,0.20]

    sc=[0]*6
    axes=[]

    for i in range(6):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(6):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=bname
    if comb == 'Yes':
        sL1=sL1+' 1st Harmonic'
    else:
        sL1=sL1+' Y+ 1st Harmonic'
    if V0 == 8080:
        sL1=sL1+' V1 >= 80 V'

    if bname == 'All-Bands':
        if comb == 'Yes':
            minCa=200.0
            maxCa=450.0
            minRa=100.0
            maxRa=7e4
            minPr=1
            maxPr=100.0
            if V0 == 8080:
                minRa=0.2
                maxRa=70
                minPr=0.7
                maxPr=100.0
        else:
            minCa=400.0
            maxCa=900.0
            minRa=100
            maxRa=7e4
            minPr=0.5
            maxPr=50.0
    elif bname == 'High-Band':
        minCa=400.0
        maxCa=700.0
        minRa=0.1
        maxRa=2.0
        minPr=0.5
        maxPr=50.0
    elif bname == 'Mid-Band':
        minCa=400.0
        maxCa=850.0
        minRa=0.2
        maxRa=10.0
        minPr=0.2
        maxPr=20.0
    elif bname == 'Low-Band':
        minCa=450
        maxCa=900.0
        minRa=0.3
        maxRa=40.0
        minPr=0.1
        maxPr=6.0

    date=[]
    fres=[[],[]]
    Ca=[[],[]]
    Ra=[[],[]]
    pw=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        date.append(datet)
        L1=float(a[3])
        inpl=int(a[24])

        if V0 == 8080 and float(a[2]) < 80.0:
            continue

        if bname != 'All-Bands':
            if L1 != L:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L) \
                    +' '+datet+' '+a[1])
                diff=diff+1
                continue

        fqp=float(a[5])
        fqm=float(a[16])

        if(L1 < 0.06 and (fqp>32e3 or fqm>32e3)):
            diff=diff+1
            continue
        if ((L1 > 0.1 and L1 < 0.5) and (fqp>10.3e3 or fqm>10.3e3)):
            diff=diff+1
            continue
        if (L1 > 1.0 and (fqp > 4e3 or fqm >4e3)):
            diff=diff+1
            continue

        if inpl == 1:
            Ra1=float(a[9])        #Ra in Ohm
            Ca1=float(a[11])       #Ca in pF

            if comb == 'Yes':
                fq2=float(a[16])
                Ra2=float(a[20])       #Ra in Ohm
                Ca2=float(a[22])       #Ca in pF
                prad1=0.5e-6*(float(a[8])**2*Ra1+float(a[19])**2*Ra2)

                fq1=0.5*(fq1+fq2)
                Ra1=Ra1+Ra2
                Ca1=Ca1*Ca2/(Ca1+Ca2)
            else:
                prad1=0.5e-6*float(a[8])**2*Ra1   #(I in mA. Pwer in W)

            fres[0].append(fq1/1.0e3)         #frequency in kHz
            Ra[0].append(np.log10(Ra1)) #log scale in kilo Ohm
            Ca[0].append(Ca1)
            pw[0].append(np.log10(prad1))    #logscale in W
        else:
            fq2=float(a[5])
            Ra2=float(a[9])        #Ra in k Ohm
            Ca2=float(a[11])       #Ca in pF

            if comb == 'Yes':
                fq1=float(a[16])
                Ra1=float(a[20])        #Ra in k Ohm
                Ca1=float(a[22])       #Ca in pF
                prad2=0.5e-6*(float(a[8])**2*Ra2+float(a[19])**2*Ra1)

                fq2=0.5*(fq1+fq2)
                Ra2=Ra1+Ra2
                Ca2=Ca1*Ca2/(Ca1+Ca2)
            else:
                prad2=0.5e-6*float(a[8])**2*Ra2   #(I in mA. Pwer in W)
            
            fres[1].append(fq2/1.0e3)
            Ra[1].append(np.log10(Ra2))
            Ca[1].append(Ca2)
            pw[1].append(np.log10(prad2))

    #end 'for loop' for reading data file

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fsmin=[min(fres[0]), min(fres[1])]
    fsmax=[max(fres[0]), max(fres[1])]

    if bname == 'All-Bands':
        xscale='log'
        fmin=2.5
        fmax=40
    else:
        xscale='linear'
        fmin=min([min(fres[0]), min(fres[1])])
        fmax=min([max(fres[0]), max(fres[1])])

    # --- calculate dipole antenna capacitance
    pi=math.pi
    pi2=2.0*pi
    La=80.0
    ra=0.10668

    xbin=int(fmax+2-fmin)
    freq=[0]*xbin
    Cvac=[0]*xbin

    for j in range(xbin):
        freq[j]=fmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        omega=pi2*freq[j]*1.0e3
        Xvac=120.0*(np.log(La/(2.0*ra))-1.0)/np.tan(pi*La/wlen)
        Cvac[j]=1.0e12/(omega*Xvac)       # vacuum dipole Capacitance
        if comb == 'No':
            Cvac[j]=2.0*Cvac[j]           #double vacuum dipole Ca for one element

    N=8   # running average window width

    #-- for dot by dot plots of data points in each bin: y-axis
    ybin=80 # for 2D binning not for 1D
    y=np.zeros((3,ybin+1))
    yCa=np.zeros(ybin)
    yRa=np.zeros(ybin)
    ypw=np.zeros(ybin)

    sc=[0]*3
    for i in range(2):
        if i == 0:
            kb=0
        else:
            kb=3

        xbin=int((fsmax[i]-fsmin[i])/bwd)
        x=[0]*(xbin+1)
        xfr=[0]*xbin
        for k in range(xbin+1):
            x[k]=fsmin[i]+float(k)*bwd
            if k > 0:
                xfr[k-1]=0.5*(x[k-1]+x[k])

        #--- determine y min & max
        Camin=min(Ca[i])
        Camax=max(Ca[i])
        Ramin=min(Ra[i])
        Ramax=max(Ra[i])
        pwmin=min(pw[i])
        pwmax=max(pw[i])
        dCa=(Camax-Camin)/ybin
        dRa=(Ramax-Ramin)/ybin
        dpw=(pwmax-pwmin)/ybin

        #--- bin size and center of bins for Ca, Ra and pwr
        for k in range(ybin+1):
            y[0][k]=Camin+float(k)*dCa
            y[1][k]=Ramin+float(k)*dRa
            y[2][k]=pwmin+float(k)*dpw
            if k > 0:
                yCa[k-1]=0.5*(y[0][k-1]+y[0][k])

                yRa[k-1]=0.5*(y[1][k-1]+y[1][k])
                yRa[k-1]=10.0**yRa[k-1]

                ypw[k-1]=0.5*(y[2][k-1]+y[2][k])
                ypw[k-1]=10.0**ypw[k-1]

        # --- calculate number of Ca data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Ca[i],fres[i],statistic='count', \
            bins=[x,y[0]])
        a=counts.statistic  # number of data points for Ca
        a=a.transpose()

        rdl=len(a[0])+1-N
        Cab2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            Cab2d[k]=moving_average.data_points_running_average(a[k], N)
        Cab2d=Cab2d.transpose()

        # --- calculate number of Ra data points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],Ra[i],fres[i],statistic='count', \
            bins=[x,y[1]])
        a=counts.statistic  # number of data points for Ra
        a=a.transpose()

        rdl=len(a[0])+1-N
        Rab2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            Rab2d[k]=moving_average.data_points_running_average(a[k], N)
        Rab2d=Rab2d.transpose()

        # --- calculate number of power points in bins [x, y]
        counts=stats.binned_statistic_2d(fres[i],pw[i],fres[i],statistic='count', \
            bins=[x,y[2]])
        a=counts.statistic  # number of data points for pwr
        a=a.transpose()

        rdl=len(a[0])+1-N
        pwb2d=np.zeros((len(a), rdl))
        for k in range(len(a)):
            pwb2d[k]=moving_average.data_points_running_average(a[k], N)
        pwb2d=pwb2d.transpose()

        # plot number of data points in each bin, onece for yCa (or yRa or ypw)
        # points at given frequency xfr[j]. The data points number is normalized
        # by maximum at each frequency
        for j in range(len(xfr[0:rdl])):
            xx=[xfr[j]]*ybin
            Cabmax=max(Cab2d[j])
            if Cabmax > 0:
                sc[0]=axes[kb].scatter(xx,yCa,c=Cab2d[j]/Cabmax,cmap=colormp,s=0.1, \
                    marker='s')

            Rabmax=max(Rab2d[j])
            if Rabmax > 0:
                sc[1]=axes[1+kb].scatter(xx,yRa,c=Rab2d[j]/Rabmax,cmap=colormp,s=0.1, \
                    marker='s')

            pwbmax=max(pwb2d[j])
            if pwbmax > 0:
                sc[2]=axes[2+kb].scatter(xx,ypw,c=pwb2d[j]/pwbmax,cmap=colormp, \
                    s=0.1,marker='s')

        #----- set axis properties for Ca ------
        #axes[kb].plot(freq, Cvac, ls='dashed', lw=0.3, color='black')
        axes[kb].set_xlim(fmin,fmax)
        axes[kb].set_xscale(xscale)
        axes[kb].set_ylim(minCa,maxCa)
        axes[kb].set_ylim(15,300)
        axes[kb].set_yscale('log')
        axes[kb].set_yticks([15,100])
        axes[kb].set_yticklabels(['15','$10^2$'])
        axes[kb].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.2,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')
        #axes[kb].yaxis.set_minor_locator(AutoMinorLocator())
        axes[kb].set_xticklabels([])

        #----- set axis properties for Ra ------
        #note: / operator cannot be applied to list but can be used for numpy array
        axes[1+kb].set_xlim(fmin,fmax)
        axes[1+kb].set_xscale(xscale)
        axes[1+kb].set_yscale('log')
        axes[1+kb].set_ylim(minRa,maxRa)
        axes[1+kb].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.2,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[1+kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[1+kb].set_xticklabels([])

        #----- set axis properties for pwr ------
        axes[2+kb].set_xlim(fmin,fmax)
        axes[2+kb].set_xscale(xscale)
        axes[2+kb].set_yscale('log')

        if bname == 'All-Bands':
            axes[2+kb].set_xticks([2.5,5,10,20,30,40])
            axes[2+kb].set_xticklabels(['2.5','5','10','20','30','40'])

        axes[2+kb].set_ylim(minPr,maxPr)
        axes[2+kb].tick_params(axis='both',which='major',length=1.2,width=0.4, \
            pad=1,labelsize=2.2,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[2+kb].tick_params(axis='both',which='minor',length=0.7,width=0.4, \
            pad=1,labelsize=0,left=True,right=True,bottom=True,top=True, \
            direction='in')
        axes[2+kb].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5, \
            labelpad=0.7)

    #ff=[2.7,33.0]
    #XX=[160,18]
    #axes[0].plot(ff, XX, ls='dashed', lw=0.3, color='black')
    #ff=[2.7,33.0]
    #RR=[27,0.39]
    #axes[1].plot(ff, RR, ls='dashed', lw=0.3, color='black')

    # --- y-axis label
    #axes[0].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.7)
    axes[0].set_ylabel('$C_{a}$ (pF)',fontsize=2.5,labelpad=0.7)
    axes[1].set_ylabel('$R_a$ (Ohm)',fontsize=2.5,labelpad=0.7)
    axes[2].set_ylabel('Dissipated Power (W)',fontsize=2.5,labelpad=0.7)

    # make y-axis tick lables invisible for right panels
    for j in range(3,6):
        axes[j].set_yticklabels([])

    # --- add text: date, inductance band, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)

    axes[0].text(1.,1.12,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
        ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(1.,1.04,sL1,fontsize=2.5,ha='center',va='bottom', \
        transform=axes[0].transAxes)
    axes[0].text(0,1.04,'Inside Plasmasphere',fontsize=2.5,ha='left',va='bottom', \
        transform=axes[0].transAxes)
    axes[3].text(1,1.04,'Outside Plasmasphere',fontsize=2.5,ha='right',va='bottom', \
        transform=axes[3].transAxes)

    # ---- color bar ----
    axins = inset_axes(axes[2], width='70%', height='8%', loc='lower left', \
    bbox_to_anchor=(0.6,-0.48,1,1), bbox_transform=axes[2].transAxes)
    cbar=fig.colorbar(sc[2], cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('Relative Number of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    return canv, fig, toolbar, fl
