#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 18 13:20:06 2019

@author: jtu
"""
import os
import numpy as np
import math
from scipy import stats

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as figtk
except ImportError:
    from matplotlib.backends.backend_tkagg import FigureCanvasTk as figtk

try:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg as navitk
except ImportError:
    from matplotlib.backends.backend_tkagg import NavigationToolbar2Tk as navitk

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#implement th edefault matplotlib key bindings
import matplotlib.ticker as ticker
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
from matplotlib.ticker import MultipleLocator

import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator

import ant_impedance
import days_of_year

#------------ plot Ra, Ca, and Prad only as scatter plots --------------------------
def plot_Ra_Ca_Prad_dots(wd, canv, fig, toolbar, fpath, bwd, L, bname, inpflag, fl):

    if len(fl) == 0:
        print('Read calculated antenna parameters ...')
        fdir=fpath.strip()
        inpflag=inpflag.strip()
        if fdir[len(fdir)-1] == '/':
            datafile=fdir+'All-Bands_InOut'+'_data.dat'
        else:
            datafile=fdir+'/All-Bands_InOut'+'_data.dat'

        with open(datafile,'r') as dataf:
            dataf.readline()
            dataf.readline()

            fl=dataf.readlines()

            dataf.close()

        print('Finished reading data')

    nrecd=len(fl)

    colormp=plt.cm.get_cmap('rainbow')

    if inpflag == 'In':
        inpl=1
    else:
        inpl=2

#------------ make plots on the tkinter window -----------------------------
    #clear figure if it exists
    if fig != None:
        fig.clf()
        
    #clear canvas if it exists
    if canv != None:
        canv.get_tk_widget().destroy()

    fig=Figure(figsize=(2.5,2.0),dpi=400)

    canv = figtk(fig, master=wd)
    canv.draw()
    canv.get_tk_widget().place(x=0,y=50)

    if toolbar != 0:
        toolbar.destroy()

    toolbar=navitk(canv, wd)
    toolbar.update()

    def on_key_press(event):
        key_press_handler(event, canv, toolbar)
        canv.mpl_connect("key_press_event", on_key_press)

    #set size, resolution, and position of axes
    xL=[0.10,0.10,0.10,0.56,0.56,0.56]
    yB=[0.66,0.40,0.14,0.66,0.40,0.14]
    wh=[0.42,0.42,0.42,0.42,0.42,0.42]
    ht=[0.23,0.23,0.23,0.23,0.23,0.23]

    sc=[0]*6
    axes=[]

    for i in range(6):
        axes.append(fig.add_axes([xL[i], yB[i], wh[i], ht[i]]))
        fig.add_axes(axes[i])

    #set line width of axes
    for ax in ['left','bottom','right','top']:
        for i in range(6):
            axes[i].spines[ax].set_linewidth(0.3)

    sL1='$L_1$ = '+str("{0:.4f}".format(L))+' H'
    if bname == 'All-Bands':
        sL1=bname

    minCa=400.0
    minRa=0.02
    minPr=0.1
    if bname == 'All-Bands':
        maxCa=900.0
        maxRa=80.0
        maxPr=50.0
    elif bname == 'High-Band':
        maxCa=350.0
        maxRa=2.0
        maxPr=100.0
    elif bname == 'Low-Band':
        maxCa=500.0
        maxRa=60.0
        maxPr=0.0
    elif bname == 'Mid-Band':
        maxCa=450.0
        maxRa=20.0
        maxPr=50.0

    date=[]
    fres=[[],[]]
    Ca=[[],[]]
    Ra=[[],[]]
    pwr=[[],[]]

    diff=0
    for k in range(0,nrecd):
        a=fl[k].split()

        datet=a[0]
        L1=float(a[3])
        fqp=float(a[5])
        fqm=float(a[16])

        inpl=int(a[24])

        if bname != 'All-Bands':
            if L1 != L:
                print('Exclude other bands:',('{:7.4f}'*2).format(L1,L)+' '+datet+' '+a[1])
                diff=diff+1
                continue

        if inpflag != 'InOut':
            if inpflag == 'In': #skip if not in plasmasphere
                if inpl == 0:
                    print('Out of the plasmasphere:','{:2d}'.format(inpl)+' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            else:
                if inpl == 1:  #skip if in the plasmasphere
                    print('In the plasmasphere:','{:2d}'.format(inpl)+' '+datet+' '+a[1])
                    diff=diff+1
                    continue
            
        date.append(datet)

        Ra1=float(a[9])/1.0e3  #Ra in k Ohm
        Ca1=float(a[11])       #Ca in pF

        Ra2=float(a[20])/1.0e3
        Ca2=float(a[22])

        prad1=0.5e-3*float(a[8])**2*Ra1  #(I in mA, Ra in 10^3 Ohm. Pwer in W)
        prad2=0.5e-3*float(a[19])**2*Ra2

        fres[0].append(fqp/1.0e3)        #frequency in kHz
        Ra[0].append(Ra1)
        Ca[0].append(Ca1)
        pwr[0].append(prad1)

        fres[1].append(fqm/1.0e3)
        Ra[1].append(Ra2)
        Ca[1].append(Ca2)
        pwr[1].append(prad2)

    #end for loop

    if diff > 0:
        print(diff,'data out of',nrecd,'not used')

    fmin1=min(fres[0])
    fmax1=max(fres[0])
    fmin2=min(fres[1])
    fmax2=max(fres[1])
    fsmin=min([fmin1,fmin2])
    fsmax=max([fmax1,fmax2])

    Cab=[[],[]]
    Rab=[[],[]]
    pwb=[[],[]]
    bcnt=[[],[]]
    fbin=[[],[]]

    # -------- bin the data ---------------
    nln=len(fres[0])
    if bname == 'All-Bands':
        # bin the data in separated three bands
        frlt=[[],[],[],[],[],[]]   #list of 6 lists
        Calt=[[],[],[],[],[],[]]
        Ralt=[[],[],[],[],[],[]]
        pwlt=[[],[],[],[],[],[]]

        for i in range(nln):
            # --- three bands for Y+ 1st harmonic
            if fres[0][i] < 5.0:
                kb=0
            elif fres[0][i] > 7.0 and fres[0][i] < 12.0:
                kb=1
            elif fres[0][i] > 20.0:
                kb=2

            frlt[kb].append(fres[0][i])
            Calt[kb].append(Ca[0][i])
            Ralt[kb].append(Ra[0][i])
            pwlt[kb].append(pwr[0][i])

            # --- three bands for Y- 1st harmonic
            if fres[1][i] < 5.0:
                kb=3
            elif fres[1][i] > 7.0 and fres[1][i] < 12.0:
                kb=4
            elif fres[1][i] > 20.0:
                kb=5

            frlt[kb].append(fres[1][i])
            Calt[kb].append(Ca[1][i])
            Ralt[kb].append(Ra[1][i])
            pwlt[kb].append(pwr[1][i])

        # --- bin the data in three bands, each for Y+ and y- monopole
        for i in range(3):
            fminlt=min(frlt[i])
            fmaxlt=max(frlt[i])

            nbin=int((fmaxlt-fminlt)/bwd)
 
            stat1=stats.binned_statistic(frlt[i], Calt[i], 'sum', bins=nbin)
            bsum1=stat1[0]
            stat1=stats.binned_statistic(frlt[i], Calt[i], 'mean', bins=nbin)
            stat2=stats.binned_statistic(frlt[i], Ralt[i], 'mean', bins=nbin)
            stat3=stats.binned_statistic(frlt[i], pwlt[i], 'mean', bins=nbin)

            stat4=stats.binned_statistic(frlt[i+3], Calt[i+3], 'sum', bins=nbin)
            bsum2=stat4[0]
            stat4=stats.binned_statistic(frlt[i+3], Calt[i+3], 'mean', bins=nbin)
            stat5=stats.binned_statistic(frlt[i+3], Ralt[i+3], 'mean', bins=nbin)
            stat6=stats.binned_statistic(frlt[i+3], pwlt[i+3], 'mean', bins=nbin)

            if i == 0:
                if inpflag == 'In':
                    lj=0
                    hj=4
                else:
                    lj=2
                    hj=0
            if i == 1:
                lj=3
                hj=7
            elif i == 2:
                if inpflag == 'In':
                    lj=12
                    hj=14
                else:
                    lj=13
                    hj=10

            for j in range(lj,nbin-hj):
                Cab[0].append(stat1[0][j])
                bcnt[0].append(bsum1[j]/stat1[0][j])
                Rab[0].append(stat2[0][j])
                pwb[0].append(stat3[0][j])

                fbin[0].append(0.5*(stat1[1][j]+stat1[1][j+1]))

                Cab[1].append(stat4[0][j])
                bcnt[1].append(bsum2[j]/stat4[0][j])
                Rab[1].append(stat5[0][j])
                pwb[1].append(stat6[0][j])

                fbin[1].append(0.5*(stat4[1][j]+stat4[1][j+1]))

        fsmin=2.5
        fsmax=40.0

        xscale='log'
    else:
        nbin=int((fsmax-fsmin)/bwd)
        stat1=stats.binned_statistic(fres[0], Ca[0], 'sum', bins=nbin)
        bsum=stat1[0]
        stat=stats.binned_statistic(fres[0], Ca[0], 'mean', bins=nbin)
        Cab[0]=stat[0]
        bcnt[0]=bsum/Cab[0]

        #Iaam=np.sqrt(pwr[0:i]/Ra[0:i])
        stat=stats.binned_statistic(fres[0], Ra[0], 'mean', bins=nbin)
        Rab[0]=stat[0]

        #-------- binned radiation power ------
        stat=stats.binned_statistic(fres[0], pwr[0], 'mean', bins=nbin)
        pwb[0]=stat[0]

        stat2=stats.binned_statistic(fres[1], Ca[1], 'sum', bins=nbin)
        bsum=stat2[0]
        stat=stats.binned_statistic(fres[1], Ca[1], 'mean', bins=nbin)
        Cab[1]=stat[0]
        bcnt[1]=bsum/Cab[1]

        #Iaam=np.sqrt(pwr[0:i]/Ra[0:i])
        stat=stats.binned_statistic(fres[1], Ra[1], 'mean', bins=nbin)
        Rab[1]=stat[0]

        #-------- binned radiation power ------
        stat=stats.binned_statistic(fres[1], pwr[1], 'mean', bins=nbin)
        pwb[1]=stat[0]

        for j in range(0,nbin):
            fbin[0][j]=0.5*(stat1[1][j]+stat1[1][j+1])
            fbin[1][j]=0.5*(stat2[1][j]+stat2[1][j+1])

        xscale='linear'

        if bname == 'High-Band' and inpflag == 'In':
            lj=6
            hj=16
        elif bname == 'High-Band' and inpflag == 'Out':
            lj=1
            hj=0
        else:
            lj=0
            hj=0

        fbin[1]=fbin[1][lj:nbin-hj]
        Cab[1]=Cab[1][lj:nbin-hj]
        Rab[1]=Rab[1][lj:nbin-hj]
        pwb[1]=pwb[1][lj:nbin-hj]
        bcnt[1]=bcnt[1][lj:nbin-hj]

        fbin[0]=fbin[0][lj:nbin-hj]
        Cab[0]=Cab[0][lj:nbin-hj]
        Rab[0]=Rab[0][lj:nbin-hj]
        pwb[0]=pwb[0][lj:nbin-hj]
        bcnt[0]=bcnt[0][lj:nbin-hj]

    # --- calculate dipole antenna capacitance and resistance in vacuum
    pi=math.pi
    pi2=2.0*pi
    La=80.0
    ra=0.10668

    xbin=int(fsmax+2-fsmin)
    freq=[0]*xbin
    Cvac=[0]*xbin

    for j in range(xbin):
        freq[j]=fsmin+float(j)

        wlen=3.0e8/(freq[j]*1.0e3)
        omega=pi2*freq[j]*1.0e3
        Xvac=120.0*(np.log(La/(2.0*ra))-1.0)/np.tan(pi*La/wlen)
        Cvac[j]=2.0e12/(omega*Xvac)       # double of vacuum dipole Capacitance

    for i in range(2):
        if i == 0:
            kb=0
        else:
            kb=3

        #----- plot antenna Ca ------
        axes[kb].scatter(fres[i], Ca[i], s=0.04, marker='+',color='black')
        axes[kb].scatter(fbin[i], Cab[i], c=bcnt[i], cmap=colormp, s=0.07, marker='s')
        axes[kb].plot(freq, Cvac, ls='dashed', lw=0.3, color='red')
        axes[kb].set_xlim(fsmin,fsmax)
        axes[kb].set_xscale(xscale)
        axes[kb].set_ylim(minCa,maxCa)
        axes[kb].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
        axes[kb].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
        #axes[kb].xaxis.set_minor_locator(AutoMinorLocator())
        axes[kb].yaxis.set_minor_locator(AutoMinorLocator())
        axes[kb].set_xticklabels([])

        # ---- plot Ra ------
        #note: / operator cannot be applied to list but can be used for numpy array
        axes[1+kb].scatter(fres[i], Ra[i], s=0.04, marker='+',color='black')
        axes[1+kb].scatter(fbin[i], Rab[i], c=bcnt[i], cmap=colormp, s=0.07, marker='s')
        axes[1+kb].set_xlim(fsmin,fsmax)
        axes[1+kb].set_xscale(xscale)
        axes[1+kb].set_yscale('log')
        axes[1+kb].set_ylim(minRa,maxRa)
        axes[1+kb].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
        axes[1+kb].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
        #axes[1+kb].xaxis.set_minor_locator(AutoMinorLocator())
        #axes[1+kb].yaxis.set_minor_locator(AutoMinorLocator())
        axes[1+kb].set_xticklabels([])
        #axes[1+kb].set_ylabel('$I_a$ (mA)',fontsize=2.2,labelpad=0.7)

        # ---- plot radiation power
        axes[2+kb].scatter(fres[i], pwr[i], s=0.04, marker='+',color='black')
        sc=axes[2+kb].scatter(fbin[i], pwb[i], c=bcnt[i], cmap=colormp, s=0.07, marker='s')
        axes[2+kb].set_xlim(fsmin,fsmax)
        axes[2+kb].set_xscale(xscale)
        axes[2+kb].set_yscale('log')

        if bname == 'All-Bands':
            axes[2+kb].set_xticks([2.5,5,10,20,30,40])
            axes[2+kb].set_xticklabels(['2.5','5','10','20','30','40'])

        axes[2+kb].set_ylim(minPr,maxPr)
        axes[2+kb].tick_params(axis='both',which='major',length=1.2,width=0.4,pad=1, \
            labelsize=2.2,left=True,right=True,bottom=True,top=True,direction='in')
        axes[2+kb].tick_params(axis='both',which='minor',length=0.7,width=0.4,pad=1, \
            labelsize=0,left=True,right=True,bottom=True,top=True,direction='in')
        #axes[2+kb].xaxis.set_minor_locator(AutoMinorLocator())
        #axes[2+kb].yaxis.set_minor_locator(AutoMinorLocator())
        axes[2+kb].set_xlabel('Resonance Frequency (kHz)',fontsize=2.5,labelpad=0.7)

    # y-axis label
    axes[0].set_ylabel('$C_a$ (pF)',fontsize=2.5,labelpad=0.7)
    axes[1].set_ylabel('$R_a$ ($10^{3}$ Ohm)',fontsize=2.5,labelpad=0.7)
    axes[2].set_ylabel('Dissipated Power (W)',fontsize=2.5,labelpad=0.7)

    # --- add text: date, inductance, Y+ / Y - 1st harmonic ----
    datemax=max(date)
    datemin=min(date)
    if inpflag == 'In':
        txt=sL1+' / In Plasmasphere'
    elif inpflag == 'Out':
        txt=sL1+' / Out of Plasmasphere'
    else:
        txt=sL1+' / Inside and Out of Plasmasphere'

    axes[0].text(1.,1.26,datemin[0:10]+'$-$'+datemax[0:10],fontsize=2.5, \
                ha='center',va='bottom',transform=axes[0].transAxes)
    axes[0].text(1.,1.15,txt,fontsize=2.5,ha='center',va='bottom', \
                transform=axes[0].transAxes)
    axes[0].text(0,1.04,'Y+ 1st Harmonic',fontsize=2.5,ha='left',va='bottom', \
                transform=axes[0].transAxes)
    axes[3].text(1,1.04,'Y- 1st Harmonic',fontsize=2.5,ha='right',va='bottom', \
                transform=axes[3].transAxes)

    # ---- vertical color bar ----
    #cbar=fig.colorbar(sc, ax=axes[3], shrink=0.9, pad=0.03, aspect=10)
    #cbar.ax.tick_params(labelsize=1.9, length=0.85, width=0.4, pad=0.4)
    #cbar.ax.set_ylabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    #cbar.outline.set_visible(False)

    # ---- color bar ----
    axins = inset_axes(axes[2], width='70%', height='8%', loc='lower left', \
        bbox_to_anchor=(0.6,-0.48,1,1), bbox_transform=axes[2].transAxes)
    cbar=fig.colorbar(sc, cax=axins, shrink=0.9, pad=0.03, aspect=10, \
        fraction=0.1, orientation='horizontal',boundaries=None)
    cbar.ax.tick_params(labelsize=1.9, length=1.3, width=0.3, pad=0.4)
    cbar.ax.set_xlabel('No. of Data Points', fontsize=2.1, labelpad=1.1)
    cbar.outline.set_visible(False)

    #if bname == 'High-Band':
    #    axes[0].scatter(fresexc, Caexc, s=0.01, marker='.',color='blue')
    #    axes[1].scatter(fresexc, Raexc, s=0.01, marker='.',color='blue')
    #    axes[2].scatter(fresexc, pwrexc, s=0.01, marker='.',color='blue')
    #if bname == 'High-Band' and inpflag == 'In':
    #    x=[23.0,27.5,32.3]
    #    y=[0]*3
    #    for i in range(3):
    #        y[i]=-x[i]+42.5
    #    axes[2].plot(x, y, lw=0.3, ls='dashed', color='red')

    return canv, fig, toolbar, fl
